/*
 * IPathDiscovery.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: elefth, Standlove
 *          Pops, TCSDEVELOPER
 */

using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines the contract for classes used to discover the path from current node to the root node.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// Implementations of this interface will be called with the current node and the current page context.
    /// The implementation is then supposed to discover the complete path from the current node to the root
    /// or return null when it can't discover the current path.
    /// </para>
    /// <para>
    /// Implementations of this class will be created in the <see cref="BreadCrumbTrailControl.RenderContents"/>
    /// method of the <see cref="BreadCrumbTrailControl"/> via the object factory.
    /// The class should then discover the path to the root node.
    /// The returned path, if any, will then be used to format the html (ie. each node will be passed
    /// to the <see cref="IPathWriter"/> for formatting with path separators in between).
    /// </para>
    /// <para>
    /// <b>Thread Safety:</b> Implementations should be thread safe in the context of the rendering process.
    /// </para>
    /// </remarks>
    public interface IPathDiscovery
    {
        /// <summary>
        /// Returns the path from the current node for the given page context.
        /// </summary>
        ///
        /// <remarks>
        /// The current node can have a null title if the current node should default to
        /// that found in the data source for this implementation. The returned nodes, however,
        /// should have a title assigned to each node. The current node's url can include
        /// query strings on both the current node that is called and in the nodes
        /// that are returned in the path. Implementors of this method should return an array
        /// where the 0-th index position is the 'root' node and the (n-1)th index position
        /// is the current node or return null if the current path could not be found.
        /// </remarks>
        ///
        /// <param name="currentNode">
        /// A non-null node representing the current page.
        /// </param>
        /// <param name="page">
        /// The non-null page context.
        /// </param>
        ///
        /// <returns>
        /// An array of <see cref="BreadCrumbNode"/> representing the path from root (0-th index)
        /// to current node ((n-1)th index) or null if path couldn't be found.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="BreadCrumbException">
        /// Any error occurs discovering the path.
        /// </exception>
        BreadCrumbNode[] GetPath(BreadCrumbNode currentNode, Page page);
    }
}

