/*
 * IPathWriter.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: elefth, Standlove
 *          Pops, TCSDEVELOPER
 */

using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines the contract for classes used to write out the discovered path.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// Implementations of this interface will be called with a writer, the path to write,
    /// and the bread crumb trail control itself. The implementation is then responsible
    /// for formatting the path in it's own way - writing the path out to the
    /// <see cref="HtmlTextWriter"/>.
    /// </para>
    /// <para>
    /// Implementations of this class will be created in the <see cref="BreadCrumbTrailControl.RenderContents"/>
    /// method of the <see cref="BreadCrumbTrailControl"/> via the object factory.
    /// </para>
    /// <para>
    /// <b>Thread Safety:</b> Implementations should be thread safe in the context of the rendering process.
    /// </para>
    /// </remarks>
    public interface IPathWriter
    {
        /// <summary>
        /// Writes out the specified path using the specified <see cref="HtmlTextWriter"/> and
        /// the attributes specified in the passed bread crumb trail control.
        /// </summary>
        ///
        /// <remarks>
        /// Any exceptions encountered should be returned from this method depending on the setting
        /// of the <see cref="BreadCrumbTrailControl.ErrorDisposition"/>:
        /// <list type="bullet">
        /// <item>
        ///     <term><see cref="ErrorDisposition.SilentlyIgnore"/></term>
        ///     <description>The method should always return null.</description>.
        /// </item>
        /// <item>
        ///     <term><see cref="ErrorDisposition.ThrowException"/></term>
        ///     <description>
        ///     The method should stop processing immediately when
        ///     an exception is encountered and that exception should be returned
        ///     (thus a 1 length array).
        ///     </description>
        /// </item>
        /// <item>
        ///     <term><see cref="ErrorDisposition.RenderException"/></term>
        ///     <description>
        ///     The method should attempt to process as much as it can and return all exceptions
        ///     that it encounters (note: the rendering of the exceptions will then occur in
        ///     the <see cref="BreadCrumbTrailControl"/>)
        ///     </description>
        /// </item>
        /// </list>
        /// </remarks>
        ///
        /// <param name="writer">
        /// A non-null writer to use.
        /// </param>
        /// <param name="nodes">
        /// The non-null, non-empty array of nodes to write.
        /// </param>
        /// <param name="parent">
        /// A non-null parent bread crumb trail control to get attributes from.
        /// </param>
        ///
        /// <returns>
        /// A null if no exceptions occur or a &gt;= 1 length array of exceptions that occurred.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nodes"/> is an empty array or contains a null element.
        /// </exception>
        BreadCrumbException[] WritePath(
            HtmlTextWriter writer, BreadCrumbNode[] nodes, BreadCrumbTrailControl parent);
    }
}


