/*
 * AbstractNodeMatcher.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: elefth, Standlove
 *          Pops, TCSDEVELOPER
 */

using System;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// An abstract implementation of the <see cref="INodeMatcher"/> interface which
    /// provides title and url type services to subclasses.
    /// </summary>
    ///
    /// <remarks>
    /// It is strongly recommended that all implementations of the <see cref="INodeMatcher"/> inherit from
    /// this abstract base. This abstract base currently, only provides title/url services,
    /// but may provide additional services in the future.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    [Serializable]
    public abstract class AbstractNodeMatcher : INodeMatcher
    {
        /// <summary>
        /// Represents the default title that a node can take if it matches this node matcher.
        /// This variable is set by the constructor, is immutable and can never be null
        /// nor an empty (trim'd) string. The title will be referenced in the associated
        /// getter method.
        /// </summary>
        private readonly string title;

        /// <summary>
        /// Represents the default url that can be used if no contextual information can be used.
        /// This variable is set by the constructor, is immutable and can never be null
        /// nor an empty (trim'd) string. The url will be referenced in the associated
        /// getter method.
        /// </summary>
        private readonly string url;

        /// <summary>
        /// Gets the default title of this node matcher.
        /// </summary>
        ///
        /// <value>
        /// A non-null, non-empty (trimmed) string representing the title.
        /// </value>
        public string Title
        {
            get
            {
                return title;
            }
        }

        /// <summary>
        /// Gets the url of this node matcher.
        /// </summary>
        ///
        /// <value>
        /// A non-null, non-empty (trimmed) string representing the url.
        /// </value>
        public string Url
        {
            get
            {
                return url;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AbstractNodeMatcher"/> class
        /// with the specified title and url.
        /// </summary>
        ///
        /// <param name="title">
        /// A non-null, non-empty (trimmed) title to use.
        /// </param>
        /// <param name="url">
        /// A non-null, non-empty (trimmed) url to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the argument is an empty trimmed string.
        /// </exception>
        protected AbstractNodeMatcher(string title, string url)
        {
            if (title == null)
            {
                throw new ArgumentNullException("title", "'title' cannot be null");
            }
            if (url == null)
            {
                throw new ArgumentNullException("url", "'url' cannot be null");
            }
            if (title.Trim().Length == 0)
            {
                throw new ArgumentException("'title' cannot be an empty trimmed string", "title");
            }
            if (url.Trim().Length == 0)
            {
                throw new ArgumentException("'url' cannot be an empty trimmed string", "url");
            }

            this.title = title;
            this.url = url;
        }

        /// <summary>
        /// Determines whether the specified node matches this node matcher.
        /// </summary>
        ///
        /// <param name="node">
        /// A non-null node to test.
        /// </param>
        ///
        /// <returns>
        /// true if the node matches, false otherwise.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="node"/> is a null reference.
        /// </exception>
        public abstract bool Matches(BreadCrumbNode node);
    }
}
