/*
 * AbstractNodeMatcher.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: elefth, Standlove
 *          Pops, TCSDEVELOPER
 */

using System;
using System.Text.RegularExpressions;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher
{
    /// <summary>
    /// Provides node matching services to a specific regex pattern.
    /// </summary>
    ///
    /// <remarks>
    /// This class will return true when the passed node's url matches the specified pattern.
    /// This class will be created by a <see cref="XmlPathDiscovery"/> object when it's creating the site map.
    /// This class will then be used by both the <see cref="XmlPathDiscovery"/> and
    /// <see cref="DynamicPathDiscovery"/> objects to determine the path to the root node(s).
    /// The <see cref="Equals"/>/<see cref="GetHashCode"/> (required by the <see cref="INodeMatcher"/>
    /// interface) are implemented as the equals/hashCode of the nodePattern/title/url string.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    [Serializable]
    public class RegexNodeMatcher : AbstractNodeMatcher
    {
        /// <summary>
        /// Represents the pattern that will be used to determine if the node matches.
        /// This variable is set by the constructor, is immutable and can never be null
        /// or an empty (trim'd) string.
        /// The pattern will be returned in the NodePattern getter.
        /// </summary>
        private readonly string nodePattern;

        /// <summary>
        /// Represents the compiled version of the nodePattern.
        /// This variable is set by the constructor, is immutable and can never be null.
        /// The pattern will be used in the matches method and
        /// is referenced by the associated getter method.
        /// </summary>
        private readonly Regex pattern;

        /// <summary>
        /// Gets the node pattern string.
        /// </summary>
        ///
        /// <value>
        /// A non-null, non-empty (trimmedd) string representing the regex pattern
        /// used to match against nodes.
        /// </value>
        public string NodePattern
        {
            get
            {
                return nodePattern;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RegexNodeMatcher"/> class
        /// with the specified title, url, and regex pattern.
        /// </summary>
        ///
        /// <param name="title">
        /// The default title of the node.
        /// </param>
        /// <param name="url">
        /// The url to use.
        /// </param>
        /// <param name="nodePattern">
        /// The node pattern to check against.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the argument is an empty trimmed string or the nodePattern
        /// is not a valid regex string.
        /// </exception>
        public RegexNodeMatcher(string title, string url, string nodePattern)
            : base(title, url)
        {
            if (nodePattern == null)
            {
                throw new ArgumentNullException("nodePattern", "'nodePattern' cannot be null");
            }
            if (nodePattern.Trim().Length == 0)
            {
                throw new ArgumentException("'nodePattern' cannot be an empty trimmed string", "nodePattern");
            }

            this.nodePattern = nodePattern;

            // this may throw AE if the pattern is invalid
            this.pattern = new Regex(nodePattern, RegexOptions.Singleline);
        }

        /// <summary>
        /// Matches the node's url against the specified pattern.
        /// </summary>
        ///
        /// <param name="node">
        /// The node to test.
        /// </param>
        ///
        /// <returns>
        /// true if the node url matches the regex pattern; otherwise, false.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="node"/> is uninitialized (contains null Url).
        /// </exception>
        public override bool Matches(BreadCrumbNode node)
        {
            if (node.Url == null)
            {
                throw new ArgumentNullException("node", "'node.Url' cannot be null");
            }

            return pattern.IsMatch(node.Url);
        }

        /// <summary>
        /// Determines whether the given object matches this node matcher.
        /// </summary>
        ///
        /// <param name="obj">
        /// The object to match against.
        /// </param>
        ///
        /// <returns>
        /// true if the given object is a <see cref="RegexNodeMatcher"/> instance
        /// and has the same NodePattern, Title, and Url; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            if (!(obj is RegexNodeMatcher))
            {
                return false;
            }

            RegexNodeMatcher o = (RegexNodeMatcher) obj;
            return ((NodePattern == o.NodePattern) && (Title == o.Title) && (Url == o.Url));
        }

        /// <summary>
        /// Returns the hash code for this object.
        /// </summary>
        ///
        /// <returns>
        /// The hashcode of the object.
        /// </returns>
        public override int GetHashCode()
        {
            return (NodePattern.GetHashCode() ^ Title.GetHashCode() ^ Url.GetHashCode());
        }
    }
}
