/*
 * TestAbstractNodeMatcher.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="AbstractNodeMatcher"/> class
    /// via <see cref="MyNodeMatcher"/> concrete class.
    /// </summary>
    [TestFixture]
    public class TestAbstractNodeMatcher
    {
        /// <summary>
        /// A sample title string to use.
        /// </summary>
        private const string title = "Sample page title";

        /// <summary>
        /// A sample url to use.
        /// </summary>
        private const string url = "http://www.topcoder.com/";

        /// <summary>
        /// Tests constructor and property getters accuracy.
        /// </summary>
        [Test]
        public void TestCtorGetters()
        {
            AbstractNodeMatcher matcher = new MyNodeMatcher(title, url);

            Assert.AreEqual(title, matcher.Title, "The Title is not initialized properly");
            Assert.AreEqual(url, matcher.Url, "The Url is not initialized properly");
        }

        /// <summary>
        /// Tests constructor given null title.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorNullTitle()
        {
            new MyNodeMatcher(null, url);
        }

        /// <summary>
        /// Tests constructor given null url.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorNullUrl()
        {
            new MyNodeMatcher(title, null);
        }

        /// <summary>
        /// Tests constructor given empty title.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorEmptyTitle()
        {
            new MyNodeMatcher(" ", url);
        }

        /// <summary>
        /// Tests constructor given empty url.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorEmptyUrl()
        {
            new MyNodeMatcher(title, " ");
        }

        /// <summary>
        /// Tests that the node matcher is serializable and deserializable.
        /// </summary>
        [Test]
        public void TestSerialization()
        {
            // serialize using BinaryFormatter to a MemoryStream
            INodeMatcher matcher = new MyNodeMatcher(title, url);
            MemoryStream stream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();
            formatter.Serialize(stream, matcher);

            // rewind the stream, and deserialize
            stream.Position = 0;
            object obj = formatter.Deserialize(stream);
            Assert.IsTrue(obj is MyNodeMatcher, "MyNodeMatcher is deserialized");

            // assert deserialized object
            INodeMatcher mm = (MyNodeMatcher) obj;
            Assert.IsFalse(matcher == mm, "Different instance");
            Assert.AreEqual(title, mm.Title, "The title is deserialized");
            Assert.AreEqual(url, mm.Url, "The url is deserialized");
        }
    }
}
