/*
 * TestDynamicPathDiscovery.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Web.UI;
using NUnit.Framework;
using NUnit.Extensions.Asp;
using NUnit.Extensions.Asp.AspTester;
using TopCoder.Util.ConfigurationManager;
using TopCoder.Web.UI.WebControl.BreadCrumb.Tests;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="DynamicPathDiscovery"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// Since GetPath requires a Session instance, an IIS server is used.
    /// The aspx file TestDynamicPathDiscovery.aspx is used to create the instance,
    /// call GetPath, and render the result in a grid.
    /// The sitemap test_files\BreadCrumbTrailWeb\XmlPathDiscovery.xml is also
    /// used for the test.
    /// The config file test_files\BreadCrumbTrailWeb\conf\ObjectFactory.xml defines
    /// the ISiteDiscovery implementation which is the XmlPathDiscovery class.
    /// </remarks>
    [TestFixture]
    public class TestDynamicPathDiscovery : WebFormTestCase
    {
        /// <summary>
        /// The path of the aspx file.
        /// </summary>
        private const string TestPath = TestCommon.AppPath + "/TestDynamicPathDiscovery.aspx";

        /// <summary>
        /// The url of the aspx used to test DynamicPathDiscovery.
        /// </summary>
        private const string TestUrl = TestCommon.IisServer + TestPath;

        /// <summary>
        /// Represents the tbTitle text box. Initialized in SetUp.
        /// </summary>
        private TextBoxTester tbTitle;

        /// <summary>
        /// Represents the tbUrl text box. Initialized in SetUp.
        /// </summary>
        private TextBoxTester tbUrl;

        /// <summary>
        /// Represents the btnGetPath button. Initialized in SetUp.
        /// </summary>
        private ButtonTester btnGetPath;

        /// <summary>
        /// Represents the lblCount label. Initialized in SetUp.
        /// </summary>
        private LabelTester lblCount;

        /// <summary>
        /// Represents the lblCurrentPath label. Initialized in SetUp.
        /// </summary>
        private LabelTester lblCurrentPath;

        /// <summary>
        /// Represents the gridPath DataGrid. Initialized in SetUp.
        /// </summary>
        private DataGridTester gridPath;

        /// <summary>
        /// Initializes the control testers.
        /// </summary>
        protected override void SetUp()
        {
            tbTitle = new TextBoxTester("tbTitle", CurrentWebForm);
            tbUrl = new TextBoxTester("tbUrl", CurrentWebForm);
            btnGetPath = new ButtonTester("btnGetPath", CurrentWebForm);
            lblCount = new LabelTester("lblCount", CurrentWebForm);
            lblCurrentPath = new LabelTester("lblCurrentPath", CurrentWebForm);
            gridPath = new DataGridTester("gridPath", CurrentWebForm);

            Browser.GetPage(TestUrl);
        }

        /// <summary>
        /// Tests GetPath by visiting the three defined root nodes in order.
        /// Since they have no common nodes, the previous path is simply discarded.
        /// </summary>
        [Test]
        public void TestRootNodes()
        {
            // R1 url
            tbTitle.Text = "First root";
            tbUrl.Text = "/R1";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("R1 root node", "1", lblCount.Text);
            AssertEquals("R1 path", new string[][]
                {new string[] {"First root", "/R1"}}, gridPath.TrimmedCells);

            // go to R2 url (with query string), R1 is discarded
            tbTitle.Text = "Second root";
            tbUrl.Text = "/R2?param=val";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("R2 root node", "1", lblCount.Text);
            AssertEquals("R2 path", new string[][]
                {new string[] {"Second root", "/R2?param=val"}}, gridPath.TrimmedCells);

            // go to R3 url (single root without any children), R2 is discarded
            tbTitle.Text = "Single root";
            tbUrl.Text = "/R3";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("R3 single root node", "1", lblCount.Text);
            AssertEquals("R3 path", new string[][]
                {new string[] {"Single root", "/R3"}}, gridPath.TrimmedCells);
        }

        /// <summary>
        /// Tests GetPath for level two nodes that each has a single path to root.
        /// There are three such nodes defined.
        /// Since each node has only one single unique path to root, that path is always
        /// used as the current path.
        /// </summary>
        [Test]
        public void TestLevelTwoNodes()
        {
            // N11 url
            tbTitle.Text = "N11 node";
            tbUrl.Text = "/R1/N11";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("N11 is below first root", "2", lblCount.Text);
            AssertEquals("N11 path", new string[][]
                {new string[] {"R1", "/R1"},
                 new string[] {"N11 node", "/R1/N11"}}, gridPath.TrimmedCells);

            // N20 url (share common root /R1 with N11)
            tbTitle.Text = "N20 node";
            tbUrl.Text = "/N20";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("N20 is below first root", "2", lblCount.Text);
            AssertEquals("N20 path", new string[][]
                {new string[] {"R1", "/R1"},
                 new string[] {"N20 node", "/N20"}}, gridPath.TrimmedCells);

            // N22 url (share no common root)
            tbTitle.Text = "N22 node";
            tbUrl.Text = "/R2/N22";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("N22 is below second root", "2", lblCount.Text);
            AssertEquals("N22 path", new string[][]
                {new string[] {"R2", "/R2"},
                 new string[] {"N22 node", "/R2/N22"}}, gridPath.TrimmedCells);
        }

        /// <summary>
        /// Tests GetPath for level three nodes that each has a single path to root.
        /// There is one such node defined.
        /// Expecting three nodes in each returned path.
        /// </summary>
        [Test]
        public void TestLevelThreeNodes()
        {
            // L1 url (with query string)
            tbTitle.Text = "L1 query";
            tbUrl.Text = "/L1?query=str";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("L1 is two levels below first root", "3", lblCount.Text);
            AssertEquals("L1 path", new string[][]
                {new string[] {"R1", "/R1"},
                 new string[] {"N11", "/R1/N11"},
                 new string[] {"L1 query", "/L1?query=str"}}, gridPath.TrimmedCells);
        }

        /// <summary>
        /// Tests GetPath for level three nodes that has multiple path to root.
        /// There are two such nodes defined: N12 and L2.
        /// Expecting the shortest path in each returned path.
        /// </summary>
        [Test]
        public void TestLevelThreeNodesMultiplePath()
        {
            // N12 has 2 path to root: N12->R1 and N12->N22->R2
            // shortest one is taken because we have no current path yet
            tbTitle.Text = "N12 shortest";
            tbUrl.Text = "/R1/N12";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("N12 shortest", "2", lblCount.Text);
            AssertEquals("N12 path", new string[][]
                {new string[] {"R1", "/R1"},
                 new string[] {"N12 shortest", "/R1/N12"}}, gridPath.TrimmedCells);

            // L2 has 2 path to root: L2->N20->R1 and L2->R2
            // unlike in XmlPathDiscovery where it took shortest L2->R2
            // DynamicPathDiscovery prefers the L2->N20->R1 because it shares 1 common node: R1
            tbTitle.Text = "L2 common";
            tbUrl.Text = "/L2?a=b";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("L2 path", "3", lblCount.Text);
            AssertEquals("L2 path", new string[][]
                {new string[] {"R1", "/R1"},
                 new string[] {"N20", "/N20"},
                 new string[] {"L2 common", "/L2?a=b"}}, gridPath.TrimmedCells);

            // go back to N12
            // the path N12->R1 is taken because it shares R1 as the common node
            tbTitle.Text = "N12 back";
            tbUrl.Text = "/R1/N12";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("N12 back", "2", lblCount.Text);
            AssertEquals("N12 path", new string[][]
                {new string[] {"R1", "/R1"},
                 new string[] {"N12 back", "/R1/N12"}}, gridPath.TrimmedCells);
        }

        /// <summary>
        /// Tests GetPath for node that has no parent (loop).
        /// Expecting null is returned.
        /// </summary>
        [Test]
        public void TestLoop()
        {
            // Loop1
            tbTitle.Text = "First loop";
            tbUrl.Text = "/loop1";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", ":null:", lblCurrentPath.Text);
            AssertEquals("Loop1 has no parent", "-1", lblCount.Text);

            // Loop2 (with query string)
            tbTitle.Text = "Second loop";
            tbUrl.Text = "/loop2?lo=op";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", ":null:", lblCurrentPath.Text);
            AssertEquals("Loop2 has no parent", "-1", lblCount.Text);
        }

        /// <summary>
        /// Tests GetPath for node that doesn't exist in the map.
        /// Expecting null is returned because we have no current path.
        /// </summary>
        [Test]
        public void TestNotExist()
        {
            tbTitle.Text = "Not exist";
            tbUrl.Text = "/NotExist";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", ":null:", lblCurrentPath.Text);
            AssertEquals("No path found", "-1", lblCount.Text);
        }

        /// <summary>
        /// Tests GetPath by visiting a root, going down one node, going down one more node,
        /// then go back up one node.
        /// </summary>
        [Test]
        public void TestGoDownAndBack()
        {
            // root
            tbTitle.Text = "Main root";
            tbUrl.Text = "/R1";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("First root", "1", lblCount.Text);
            AssertEquals("First path", new string[][]
                {new string[] {"Main root", "/R1"}}, gridPath.TrimmedCells);

            // go down child
            tbTitle.Text = "Go direct child";
            tbUrl.Text = "/N20?a=b";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("Second node", "2", lblCount.Text);
            AssertEquals("Direct child", new string[][]
                {new string[] {"Main root", "/R1"},
                 new string[] {"Go direct child", "/N20?a=b"}}, gridPath.TrimmedCells);

            // go down child
            tbTitle.Text = "Go grandchild";
            tbUrl.Text = "/L2?c=d";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("Third node", "3", lblCount.Text);
            AssertEquals("Grandchild", new string[][]
                {new string[] {"Main root", "/R1"},
                 new string[] {"Go direct child", "/N20?a=b"},
                 new string[] {"Go grandchild", "/L2?c=d"}}, gridPath.TrimmedCells);

            // go back up
            tbTitle.Text = "Back up";
            tbUrl.Text = "/N20?z=z";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("Back to second node", "2", lblCount.Text);
            AssertEquals("Back child", new string[][]
                {new string[] {"Main root", "/R1"},
                 new string[] {"Back up", "/N20?z=z"}}, gridPath.TrimmedCells);

        }

        /// <summary>
        /// Tests GetPath by visiting a known node (to have at least one current path),
        /// then visit some new nodes that are not on the site map,
        /// then go to some known node.
        /// The new nodes should be added dynamically.
        /// </summary>
        [Test]
        public void TestDynamic()
        {
            // root
            tbTitle.Text = "Main root";
            tbUrl.Text = "/R1";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("First root", "1", lblCount.Text);
            AssertEquals("First path", new string[][]
                {new string[] {"Main root", "/R1"}}, gridPath.TrimmedCells);

            // go to some dynamic node
            tbTitle.Text = "Dynamic node";
            tbUrl.Text = "/Dynamic?y=x";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("Dynamic count", "2", lblCount.Text);
            AssertEquals("Dynamic path", new string[][]
                {new string[] {"Main root", "/R1"},
                 new string[] {"Dynamic node", "/Dynamic?y=x"}}, gridPath.TrimmedCells);

            // more dynamic node
            tbTitle.Text = "";
            tbUrl.Text = "/SecondDynamic";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("Second dynamic", "3", lblCount.Text);
            AssertEquals("Second dynamic path", new string[][]
                {new string[] {"Main root", "/R1"},
                 new string[] {"Dynamic node", "/Dynamic?y=x"},
                 new string[] {"", "/SecondDynamic"}}, gridPath.TrimmedCells);

            // go to some known node: R1->N20->L2 (R1 is the common node)
            tbTitle.Text = "Go L2";
            tbUrl.Text = "/L2?a";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("Back to known node", "3", lblCount.Text);
            AssertEquals("Back to L2 path", new string[][]
                {new string[] {"Main root", "/R1"},
                 new string[] {"N20", "/N20"},
                 new string[] {"Go L2", "/L2?a"}}, gridPath.TrimmedCells);
        }

        /// <summary>
        /// Tests GetPath by visiting a known node (to have at least one current path),
        /// then visit two new dynamic nodes, go back to first dynamic node.
        /// The last command should compare against the url, instead of using INodeMatcher.
        /// </summary>
        [Test]
        public void TestGoBackDynamic()
        {
            // root
            tbTitle.Text = "Main root";
            tbUrl.Text = "/R1";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("First root", "1", lblCount.Text);
            AssertEquals("First path", new string[][]
                {new string[] {"Main root", "/R1"}}, gridPath.TrimmedCells);

            // go to some dynamic node
            tbTitle.Text = "Dynamic node";
            tbUrl.Text = "/Dynamic?y=x";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("Dynamic count", "2", lblCount.Text);
            AssertEquals("Dynamic path", new string[][]
                {new string[] {"Main root", "/R1"},
                 new string[] {"Dynamic node", "/Dynamic?y=x"}}, gridPath.TrimmedCells);

            // more dynamic node
            tbTitle.Text = "";
            tbUrl.Text = "/SecondDynamic";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("Second dynamic", "3", lblCount.Text);
            AssertEquals("Second dynamic path", new string[][]
                {new string[] {"Main root", "/R1"},
                 new string[] {"Dynamic node", "/Dynamic?y=x"},
                 new string[] {"", "/SecondDynamic"}}, gridPath.TrimmedCells);

            // go back to first dynamic node
            tbTitle.Text = "Different title";
            tbUrl.Text = "/Dynamic?y=x";
            btnGetPath.Click();
            AssertEquals("CurrentPath Session", "CurPath", lblCurrentPath.Text);
            AssertEquals("Back to first dynamic", "2", lblCount.Text);
            AssertEquals("Back to dynamic path", new string[][]
                {new string[] {"Main root", "/R1"},
                 new string[] {"Different title", "/Dynamic?y=x"}}, gridPath.TrimmedCells);
        }

        /// <summary>
        /// Tests GetPath given null/uninitialized node (Url is null).
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestGetPathNullNode()
        {
            new DynamicPathDiscovery().GetPath(
                new BreadCrumbNode(), new Page());
        }

        /// <summary>
        /// Tests GetPath given null page.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestGetPathNullPage()
        {
            new DynamicPathDiscovery().GetPath(
                new BreadCrumbNode("title", "url"), null);
        }

        /// <summary>
        /// Tests GetPath given page that has no Session.
        /// Expecting BreadCrumbException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(BreadCrumbException))]
        public void TestGetPathPageNoSession()
        {
            new DynamicPathDiscovery().GetPath(
                new BreadCrumbNode("title", "url"), new Page());
        }
    }
}

