/*
 * TestNodePair.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="NodePair"/> class.
    /// </summary>
    [TestFixture]
    public class TestNodePair
    {
        /// <summary>
        /// A sample node.
        /// </summary>
        private BreadCrumbNode node = new BreadCrumbNode("Test page title", "http://test/");

        /// <summary>
        /// A sample INodeMatcher.
        /// </summary>
        private INodeMatcher matcher = new MyNodeMatcher("Match title", "http://matcher");

        /// <summary>
        /// Tests constructor and property getters accuracy.
        /// </summary>
        [Test]
        public void TestCtorGetters()
        {
            NodePair pair = new NodePair(node, matcher);

            Assert.AreEqual(node.Title, pair.Node.Title, "The Node.Title is not initialized properly");
            Assert.AreEqual(node.Url, pair.Node.Url, "The Node.Url is not initialized properly");
            Assert.AreSame(matcher, pair.NodeMatcher, "The NodeMatcher is not initialized properly");
        }

        /// <summary>
        /// Tests constructor given null/uninitialized node (node.Url is null).
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorNullNode()
        {
            new NodePair(new BreadCrumbNode(), matcher);
        }

        /// <summary>
        /// Tests constructor given node with null title.
        /// Expecting that the returned node use the matcher's title.
        /// </summary>
        [Test]
        public void TestCtorNullNodeTitle()
        {
            NodePair pair = new NodePair(new BreadCrumbNode(null, "http://url"), matcher);

            Assert.IsNotNull(pair.Node.Title, "Title should not be null");
            Assert.AreEqual(matcher.Title, pair.Node.Title, "Matcher's Title is used");
            Assert.AreEqual("http://url", pair.Node.Url, "Node's Url is used");
            Assert.AreSame(matcher, pair.NodeMatcher, "The NodeMatcher is not initialized properly");
        }

        /// <summary>
        /// Tests constructor given node with empty title.
        /// Expecting that matcher's title is used because empty title
        /// is converted to null by BreadCrumbNode.
        /// </summary>
        [Test]
        public void TestCtorEmptyTitle()
        {
            NodePair pair = new NodePair(new BreadCrumbNode(" ", "http://urlempty"), matcher);

            Assert.AreEqual(matcher.Title, pair.Node.Title, "Empty Title is converted to null");
            Assert.AreEqual("http://urlempty", pair.Node.Url, "Node's Url is used");
            Assert.AreSame(matcher, pair.NodeMatcher, "The NodeMatcher is not initialized properly");
        }

        /// <summary>
        /// Tests constructor given null matcher.
        /// Expecting that node is returned as is.
        /// </summary>
        [Test]
        public void TestCtorNullMatcher()
        {
            NodePair pair = new NodePair(node, null);

            Assert.AreEqual(node.Title, pair.Node.Title, "Node's Title is used");
            Assert.AreEqual(node.Url, pair.Node.Url, "Node's Url is used");
            Assert.IsNull(pair.NodeMatcher, "No matcher");
        }

        /// <summary>
        /// Tests constructor given node with null title and null matcher.
        /// Expecting that the returned node has empty title.
        /// </summary>
        [Test]
        public void TestCtorNullNodeTitleNullMatcher()
        {
            NodePair pair = new NodePair(new BreadCrumbNode(null, "http://allnull"), null);

            Assert.IsNull(pair.Node.Title, "Null title is returned");
            Assert.AreEqual("http://allnull", pair.Node.Url, "Node's Url is used");
            Assert.IsNull(pair.NodeMatcher, "No matcher");
        }

        /// <summary>
        /// Tests that NodePair is serializable and deserializable.
        /// </summary>
        [Test]
        public void TestSerialization()
        {
            // serialize using BinaryFormatter to a MemoryStream
            NodePair pair = new NodePair(node, matcher);
            MemoryStream stream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();
            formatter.Serialize(stream, pair);

            // rewind the stream, and deserialize
            stream.Position = 0;
            object obj = formatter.Deserialize(stream);
            Assert.IsTrue(obj is NodePair, "NodePair is deserialized");

            // assert deserialized object
            NodePair nn = (NodePair) obj;
            Assert.IsFalse(pair == nn, "Different string instances");
            Assert.IsFalse(pair.NodeMatcher == nn.NodeMatcher, "Different matcher instance");
            Assert.AreEqual(node.Title, nn.Node.Title, "The node's title is deserialized");
            Assert.AreEqual(node.Url, nn.Node.Url, "The node's url is deserialized");
            Assert.AreEqual(matcher.Title, nn.NodeMatcher.Title, "The matcher's title is deserialized");
            Assert.AreEqual(matcher.Url, nn.NodeMatcher.Url, "The matcher's url is deserialized");
        }
    }
}
