/*
 * TestBreadCrumbException.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="BreadCrumbException"/> class.
    /// </summary>
    [TestFixture]
    public class TestBreadCrumbException
    {
        /// <summary>
        /// A sample error message to use.
        /// </summary>
        private const string errMessage = "Error writing bread crumb trail";

        /// <summary>
        /// A sample inner exceptions to use.
        /// </summary>
        private static Exception inner = new ArgumentException();

        /// <summary>
        /// Tests constructor() accuracy.
        /// </summary>
        [Test]
        public void TestCtor()
        {
            // instantiate using default ctor
            BreadCrumbException e = new BreadCrumbException();

            // assert default error message and inner exception is null
            Assert.IsNotNull(e.Message, "The error message is initialized to default");
            Assert.IsNull(e.InnerException, "No inner exception");
        }

        /// <summary>
        /// Tests constructor(string) accuracy.
        /// </summary>
        [Test]
        public void TestCtorString()
        {
            // instantiate with a sample error message
            BreadCrumbException e = new BreadCrumbException(errMessage);

            // assert error message is correct and inner exception is null
            Assert.AreEqual(errMessage, e.Message, "The error message is incorrect");
            Assert.IsNull(e.InnerException, "No inner exception");
        }

        /// <summary>
        /// Tests constructor(string) given null error message.
        /// Expecting a default error message in the exception.
        /// </summary>
        [Test]
        public void TestCtorStringNull()
        {
            // instantiate with null error message
            BreadCrumbException e = new BreadCrumbException(null);

            // assert default error message and inner exception is null
            Assert.IsNotNull(e.Message, "The error message is initialized to default");
            Assert.IsNull(e.InnerException, "No inner exception");
        }

        /// <summary>
        /// Tests constructor(string) given empty error message.
        /// Expecting the empty error message is accepted.
        /// </summary>
        [Test]
        public void TestCtorStringEmpty()
        {
            // instantiate with empty error message
            BreadCrumbException e = new BreadCrumbException(" ");

            // assert empty error message and inner exception is null
            Assert.AreEqual(" ", e.Message, "The error message is empty");
            Assert.IsNull(e.InnerException, "No inner exception");
        }

        /// <summary>
        /// Tests constructor(string, Exception) accuracy.
        /// </summary>
        [Test]
        public void TestCtorStringException()
        {
            // instantiate with a sample error message and inner exception
            BreadCrumbException e = new BreadCrumbException(errMessage, inner);

            // assert error message and inner exception is correct
            Assert.AreEqual(errMessage, e.Message, "The error message is incorrect");
            Assert.AreSame(inner, e.InnerException, "Inner exception is incorrect");
        }

        /// <summary>
        /// Tests constructor(string, Exception) with a null error message and null inner exceptions.
        /// Expecting a default error message in the exception and null inner exceptions.
        /// </summary>
        [Test]
        public void TestCtorStringExceptionNull()
        {
            // instantiate with null error message and inner exceptions
            BreadCrumbException e = new BreadCrumbException(null, null);

            // assert error message is initialized to default message and inner exceptions are null
            Assert.IsNotNull(e.Message, "The error message is the default message");
            Assert.IsNull(e.InnerException, "No inner exception");
        }

        /// <summary>
        /// Tests constructor(string, Exception) with an empty error message.
        /// Expecting the empty error message is accepted.
        /// </summary>
        [Test]
        public void TestCtorStringExceptionEmpty()
        {
            // instantiate with an empty error message
            BreadCrumbException e = new BreadCrumbException(" ", inner);

            // assert error message and inner exception are correct
            Assert.AreEqual(" ", e.Message, "The error message is empty");
            Assert.AreSame(inner, e.InnerException, "Inner exception is incorrect");
        }

        /// <summary>
        /// Tests that the exception is serializable and deserializable.
        /// Serializing will fail if the exception is not marked with Serializable attribute.
        /// Deserializing will fail if the exception doesn't have the constructor that accepts
        /// SerializationInfo and StreamingContext parameters.
        /// </summary>
        [Test]
        public void TestSerializable()
        {
            // serialize using BinaryFormatter to a MemoryStream
            MemoryStream stream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();
            formatter.Serialize(stream, new BreadCrumbException(errMessage, inner));

            // rewind the stream, and deserialize
            stream.Position = 0;
            object obj = formatter.Deserialize(stream);
            Assert.IsTrue(obj is BreadCrumbException, "Exception is deserialized");

            // assert deserialized exception
            BreadCrumbException e = (BreadCrumbException) obj;
            Assert.AreEqual(errMessage, e.Message, "The error message is deserialized");
            Assert.IsTrue(e.InnerException.GetType() == inner.GetType(), "The inner exception type is incorrect");
            Assert.IsFalse(e.InnerException == inner, "It's not the same instance reference");
        }

        /// <summary>
        /// Tests <see cref="BreadCrumbException"/> inherits from <see cref="ApplicationException"/>.
        /// </summary>
        [Test]
        public void TestInheritance()
        {
            try
            {
                throw new BreadCrumbException(errMessage, inner);
            }
            catch (Exception e)
            {
                // ok only if it's a direct subclass of ApplicationException
                Assert.AreEqual(typeof(ApplicationException), e.GetType().BaseType,
                    "BreadCrumbException is not a direct subclass of ApplicationException");
            }
        }
    }
}
