/*
 * TestBreadCrumbNode.cs
 *
 * Bread Crumb Trail Control
 * Version: 2.0
 *
 * Copyright (c) 2004, 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Standlove, TCSDEVELOPER
 */

using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="BreadCrumbNode"/> struct.
    /// </summary>
    [TestFixture]
    public class TestBreadCrumbNode
    {
        /// <summary>
        /// A sample title.
        /// </summary>
        private const string sampleTitle = "Sample Title Test";

        /// <summary>
        /// A sample url.
        /// </summary>
        private const string sampleUrl = "http://sample.url.com/page?a=5&b=7";

        /// <summary>
        /// Tests constructor and property getters accuracy.
        /// </summary>
        [Test]
        public void TestCtorGetters()
        {
            BreadCrumbNode node = new BreadCrumbNode(sampleTitle, sampleUrl);

            Assert.AreEqual(sampleTitle, node.Title, "The Title is not initialized properly");
            Assert.AreEqual(sampleUrl, node.Url, "The Url is not initialized properly");
        }

        /// <summary>
        /// Tests constructor and property getters given null title.
        /// Expecting that the null title is accepted.
        /// </summary>
        [Test]
        public void TestCtorNullTitle()
        {
            BreadCrumbNode node = new BreadCrumbNode(null, sampleUrl);

            Assert.IsNull(node.Title, "The Title should be null");
            Assert.AreEqual(sampleUrl, node.Url, "The Url is not initialized properly");
        }

        /// <summary>
        /// Tests constructor and property getters given empty string title.
        /// Expecting that null title is stored.
        /// </summary>
        [Test]
        public void TestCtorEmptyTitle()
        {
            BreadCrumbNode node = new BreadCrumbNode(" ", sampleUrl);

            Assert.IsNull(node.Title, "The Title should be converted to null");
            Assert.AreEqual(sampleUrl, node.Url, "The Url is not initialized properly");
        }

        /// <summary>
        /// Tests constructor and property getters given null url.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorNullUrl()
        {
            new BreadCrumbNode(sampleTitle, null);
        }

        /// <summary>
        /// Tests constructor and property getters given empty url.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorEmptyUrl()
        {
            new BreadCrumbNode(sampleTitle, " ");
        }

        /// <summary>
        /// Tests using default constructor.
        /// Expecting that all elements are initialized to null.
        /// </summary>
        [Test]
        public void TestDefaultCtor()
        {
            BreadCrumbNode node = new BreadCrumbNode();
            Assert.IsNull(node.Title, "Title should be null");
            Assert.IsNull(node.Url, "Url should be null");
        }

        /// <summary>
        /// Tests that BreadCrumbNode is serializable and deserializable.
        /// </summary>
        [Test]
        public void TestSerialization()
        {
            // serialize using BinaryFormatter to a MemoryStream
            BreadCrumbNode node = new BreadCrumbNode(sampleTitle, sampleUrl);
            MemoryStream stream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();
            formatter.Serialize(stream, node);

            // rewind the stream, and deserialize
            stream.Position = 0;
            object obj = formatter.Deserialize(stream);
            Assert.IsTrue(obj is BreadCrumbNode, "BreadCrumbNode is deserialized");

            // assert deserialized object
            BreadCrumbNode bb = (BreadCrumbNode) obj;
            Assert.AreEqual(sampleTitle, bb.Title, "The title is deserialized");
            Assert.AreEqual(sampleUrl, bb.Url, "The url is deserialized");
        }
    }
}
